/*
 * Copyright (c) 2016, Sangoma Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contributors: Pushkar Singh <psingh@sangoma.com>
 *
 */

/********************************************************************************/
/*                                                                              */
/*                           libraries inclusion                                */
/*                                                                              */
/********************************************************************************/
#include "sample_tapping.h"

/*===================================================
 * ISDN TAP CLI COMMAND
 *==================================================*/
/**********************************************************************************
 * Fun   : sangoma_freetdm_isdn_tap_statistics()
 *
 * Desc  : To get isdn tapping statistics
 *
 * Ret   : int
**********************************************************************************/
int sangoma_freetdm_isdn_tap_statistics(void)
{
	char cmd[512];
	char *rply;

	sprintf(cmd, "sangoma_isdn_tap statistics");

	rply = ftdm_api_execute(cmd);
	if (rply) {
		ftdm_log(FTDM_LOG_DEBUG, "\n%s", rply);
		ftdm_free(rply);
	}

	return 0;
}

/**********************************************************************************
 * Fun   : sangoma_freetdm_exec_io_stats()
 *
 * Desc  : To get span IO stats
 *
 * Ret   : void
**********************************************************************************/
void sangoma_freetdm_exec_io_stats(const char* cmd, ftdm_span_t * span)
{
	ftdm_iterator_t *chaniter = NULL;
	ftdm_iterator_t *curr = NULL;
	ftdm_channel_t *ftdmchan = NULL;

	if ((NULL == cmd) || (NULL == span)) {
		ftdm_log(FTDM_LOG_ERROR, "Error Enabling IO Stats..invalid span \n");
		return;
	}

	chaniter = ftdm_span_get_chan_iterator(span, NULL);
	for (curr = chaniter; curr; curr = ftdm_iterator_next(curr)) {
		ftdmchan = (ftdm_channel_t*)ftdm_iterator_current(curr);
		sangoma_freetdm_exec_io_command(cmd, ftdmchan);
	}
	ftdm_iterator_free(chaniter);
}

/**********************************************************************************
 * Fun   : sangoma_freetdm_exec_io_command()
 *
 * Desc  : To enable/disable channel specific IO statistics
 *
 * Ret   : int
**********************************************************************************/
/* This API will be used to enable/disable IOSTATS on ftdm channels */
void sangoma_freetdm_exec_io_command(const char *cmd, ftdm_channel_t *fchan)
{
	int enable = 0;
	ftdm_channel_iostats_t stats;
	char buffer[1024] = {0};
	int len = 0;

	if (!fchan || !cmd)
		return;

	if (!strcasecmp("enable", cmd)) {
		enable = 1; 
		ftdm_channel_command(fchan, FTDM_COMMAND_SWITCH_IOSTATS, &enable);
	} else if (!strcasecmp("disable", cmd)) {
		enable = 0; 
		ftdm_channel_command(fchan, FTDM_COMMAND_SWITCH_IOSTATS, &enable);
	} else if (!strcasecmp("flush", cmd)) {
		ftdm_channel_command(fchan, FTDM_COMMAND_FLUSH_IOSTATS, NULL);
	} else {
		ftdm_channel_command(fchan, FTDM_COMMAND_GET_IOSTATS, &stats);
		len = len + sprintf(buffer, "-- IO statistics for channel %d:%d --\n",
				ftdm_channel_get_span_id(fchan), ftdm_channel_get_id(fchan));
		len = len + sprintf(buffer+len, "Rx errors: %u\n", stats.rx.errors);
		len = len + sprintf(buffer+len, "Rx queue size: %u\n", stats.rx.queue_size);
		len = len + sprintf(buffer+len, "Rx queue len: %u\n", stats.rx.queue_len);
		len = len + sprintf(buffer+len, "Rx count: %lu\n", (long int)stats.rx.packets);
		len = len + sprintf(buffer+len, "Tx errors: %u\n", stats.tx.errors);
		len = len + sprintf(buffer+len, "Tx queue size: %u\n", stats.tx.queue_size);
		len = len + sprintf(buffer+len, "Tx queue len: %u\n", stats.tx.queue_len);
		len = len + sprintf(buffer+len, "Tx count: %lu\n", (long int)stats.tx.packets);
		len = len + sprintf(buffer+len, "Tx idle: %lu\n", (long int)stats.tx.idle_packets);

		ftdm_log(FTDM_LOG_DEBUG, "%s\n", buffer);

		//fprintf(stderr,"%s\n", buffer);
		ftdm_log(FTDM_LOG_DEBUG, "%s\n",buffer);
	}
}

/**********************************************************************************
 * Fun   : sangoma_freetdm_set_span_sig_queuesize()
 *
 * Desc  : To set span Queue size
 *
 * Ret   : void
**********************************************************************************/
void sangoma_freetdm_set_span_sig_queuesize(ftdm_span_t * span, int rxsize)
{
	ftdm_iterator_t *chaniter = NULL;
	ftdm_iterator_t *curr = NULL;
	ftdm_channel_t *ftdmchan = NULL;

	if (!rxsize || (NULL == span)) {
		ftdm_log(FTDM_LOG_ERROR, "Error Setting RxQueue size ..invalid span/rxsize \n");
		return;
	}

	chaniter = ftdm_span_get_chan_iterator(span, NULL);
	for (curr = chaniter; curr; curr = ftdm_iterator_next(curr)) {
		ftdmchan = (ftdm_channel_t*)ftdm_iterator_current(curr);
		if (SNG_TAP_FTDM_IS_DCHAN(ftdmchan))
			sangoma_freetdm_set_chan_sig_rx_queuesize(ftdmchan, rxsize);
	}
	ftdm_iterator_free(chaniter);
}

/**********************************************************************************
 * Fun   : sangoma_freetdm_set_chan_sig_rx_queuesize()
 *
 * Desc  : To set signalling channel queue size
 *
 * Ret   : void
**********************************************************************************/
void sangoma_freetdm_set_chan_sig_rx_queuesize(ftdm_channel_t *fchan, int rxsize)
{
	ftdm_status_t   ret_status;

	if (!fchan) return;

	if (!rxsize) return;

	if (SNG_TAP_FTDM_IS_DCHAN(fchan)) {

		fprintf(stderr,"Setting Rx Queue size for channel %d:%d --\n", ftdm_channel_get_span_id(fchan), ftdm_channel_get_id(fchan));

		ret_status = ftdm_channel_command(fchan, FTDM_COMMAND_SET_RX_QUEUE_SIZE, &rxsize);

		if (ret_status != FTDM_SUCCESS) {
			ftdm_log(FTDM_LOG_ERROR, "Failed to set Rx Queue size[%d]", rxsize);
		}
	}
}
/*********************************************************************************/

/****************** Logging module helper function *******************************/
/**********************************************************************************
 * Fun   : sangoma_freetdm_cut_path()
 *
 * Desc  : To retireve the file name for the file patch passed to function
 *
 * Ret   : char *(file name)
**********************************************************************************/
const char* sangoma_freetdm_cut_path(const char *file_path)
{
        const char *p, *ret = file_path;
        const char delims[] = "/\\";
        const char *i;

        if (file_path) {
                for (i = delims; *i; i++) {
                        p = file_path;
                        while ((p = strchr(p, *i)) != 0) {
                                ret = ++p;
                        }
                }
                return ret;
        } else {
                return NULL;
        }
}
/*********************************************************************************/

/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
 */

